//*************************************************
// CORSO PROGRAMMAZIONE PICMICRO
// Approfondimenti pratici
// http://www.settorezero.com
//
// (C) Bernardo Giovanni
//
// modulo: main.c
// autore: Bernardo Giovanni
// data:  13/11/11
// descrizione: Interfacciamento ADXL345
// picmicro: PIC12F1822
// clock: 32MHz, interno
//
// E' VIETATA LA RIPUBBLICAZIONE SU ALTRI SITI,
// FORUM O ALTRI MEZZI
//*************************************************

#define  _XTAL_FREQ 32000000
#include <pic.h>
#include "usart12f.c"
#include "i2c.c"

// Config word
// ho selezionato oscillatore interno (FOSC_INTOSC). Di conseguenza RA5 usabile come I/O
// disabilitato il clock in uscita (CLKOUTEN_OFF). Di conseguenza RA4 usabile come I/O
// ho attivato il PLL (PLLEN_ON).
__CONFIG (FOSC_INTOSC & WDTE_OFF & PWRTE_ON & MCLRE_OFF & CP_OFF & CPD_OFF & BOREN_OFF & CLKOUTEN_OFF & IESO_OFF & FCMEN_OFF);
__CONFIG (WRT_OFF & PLLEN_ON & STVREN_OFF & BORV_19 & LVP_OFF);

#define ADXL_ADDRESS	0x53   // Indirizzo a 7 bit ADXL345 con pin 12 messo a massa
unsigned char tempBuffer[6];
signed int x, y, z;
 
void main(void)
	{
	/*
	Settaggio oscillatore
	bits 6-3 : 8MHz => 32MHz dato che PLLON
	bits 1-0 : il clock  determinato da config word, non si puo cambiare a runtime
	*/
	OSCCON=0b11110000; 

	ANSELA=0; // tutti io come digitali
	ADON=0; // modulo AD spento
	WPUA=0; // pullup disabilitate

	// Attendo che il clock sia pronto (HFIOFR)
	while (!HFIOFR) {continue;}
	
	// Inizializzo l'UART
	uart_init();
	// Inizializzo l'I2C
	I2cInitMaster();
	
	// Avvio l'ADXL345
	I2cStart();
	I2cWriteMaster(ADXL_ADDRESS<<1); // invio indirizzo ADXL345 in scrittura (bit0 a 0)
	I2cWriteMaster(0x2D);
	I2cWriteMaster(8);
	I2cStop();
	
	__delay_ms(5);
	
	while(1)
		{
		I2cBulkRead(ADXL_ADDRESS,0x32,6,tempBuffer);
		x = (int)tempBuffer[1] << 8 | tempBuffer[0];   
		y = (int)tempBuffer[3] << 8 | tempBuffer[2];
		z = (int)tempBuffer[5] << 8 | tempBuffer[4];

		printf("%d,%d,%d\n\r",x,y,z);
		
		__delay_ms(250);
		} // while

	} // main
	